local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )
local UH = UnderHood
local AceEvent = LibStub( "AceEvent-3.0" )
local Threat = LibStub( "Threat-2.0" )
local _G = _G
local UnitExists = UnitExists
local UnitCanAttack = UnitCanAttack
local UnitGUID = UnitGUID

local Provider = {}
Provider.__index = Provider
Provider.name = L["Threat"]
Provider.supportedUnits = { "player" }
Provider.manageVisibility = true

function Provider:Create( bar )
	local self = {}

	setmetatable( self, Provider )
	AceEvent:Embed( self )

	self.playerGUID = UnitGUID( "player" )
	self.bar = bar

	return self
end

function Provider:UnitSupported( unit )
	return unit == "player"
end

function Provider:IsBarVisible()
	return self.targetGUID and true or false
end

function Provider:OnEnterCombat()
	self.inCombat = true
	self.bar:Show()
	
	self:Update()
end

function Provider:OnLeaveCombat()
	self.inCombat = nil
	self.bar:Hide()
end

function Provider:Connect()
	Threat.RegisterCallback( self, "ThreatUpdated", "ThreatUpdated" )
	self:RegisterEvent( "PLAYER_TARGET_CHANGED" )

	self:PLAYER_TARGET_CHANGED()
end

function Provider:Disconnect()
	self:UnregisterAllEvents()
	Threat.UnregisterCallback( self, "ThreatUpdated" );
end

function Provider:Update()
	local r,g,b = unpack( UH.db.profile.colors.unit.hostile )

	self.bar:SetColor( r, g, b, 1 )

	if self.targetGUID then
		self.bar:Show()
		self.selfThreat = Threat:GetThreat( self.playerGUID, self.targetGUID )
	else
		self.bar:Hide()
		self.selfThreat = 0
	end

	self:SetValue()
end

function Provider:ThreatUpdated( event, srcGUID, dstGUID, threat )
	if self.targetGUID and dstGUID == self.targetGUID then
		if srcGUID == self.playerGUID or srcGUID == UnitGUID( "targettarget" ) then
			self:SetValue()
		end
	end
end

function Provider:PLAYER_TARGET_CHANGED()
	if UnitExists( "target" ) then
		self.targetGUID = UnitGUID( "target" )
	else
		self.targetGUID = nil
	end

	self:Update()
end

function Provider:SetValue()
	local threat = self.targetGUID and Threat:GetThreat( self.playerGUID, self.targetGUID ) or 0
	local maxThreat = self.targetGUID and Threat:GetMaxThreatOnTarget( self.targetGUID ) or 0

	if maxThreat == 0 then
		threat = 0
	else
		threat = threat/maxThreat
	end

	self.bar:SetValue( threat )
end

UnderHood_Bars:RegisterProvider( Provider )
